/*$Id: d_res.cc,v 17.21 2000/05/02 09:32:08 al Exp $ -*- C++ -*-
 * functions for resistor.
 * y.x = amps,  y.f0 = volts, ev = y.f1 = ohms
 * m.x = volts, m.c0 = amps, acg = m.c1 = mhos.
 */
#include "d_res.h"
/*--------------------------------------------------------------------------*/
void DEV_RESISTANCE::precalc()
{
  if (value() == 0. && !has_common()){
    error(bPICKY, long_label() + ": short circuit\n");
  }
  _y0.f0 = LINEAR;
  _y0.f1 = (value() != 0.) ? value() : OPT::shortckt;
  _y1 = _y0;
  _m0.c1 = 1./_y0.f1;
  _m0.c0 = 0.;
  _m1 = _m0;
  assert(_loss0 == 0.);
  assert(_loss1 == 0.);
  set_constant(!has_tr_eval());
  set_converged(!has_tr_eval());
}
/*--------------------------------------------------------------------------*/
void DEV_RESISTANCE::dc_begin()
{
  if (!has_tr_eval()){
    assert(_y0.f0 == LINEAR);
    assert(_y0.f1 == value() || _y0.f1 == OPT::shortckt);
    assert(conchk(_m0.c1, 1./_y0.f1));
    assert(_m0.c0 == 0.);
    assert(_m1 == _m0);
    assert(_loss0 == 0.);
    assert(_loss1 == 0.);
  }
}
/*--------------------------------------------------------------------------*/
bool DEV_RESISTANCE::do_tr()
{
  {if (has_tr_eval()){
    _m0.x = tr_involts_limited();
    _y0.x = _m0.c0 + _m0.c1 * _m0.x;
    tr_eval();
    assert(_y0.f0 != LINEAR);
    if (_y0.f1 == 0.){
      untested();
      error(bPICKY, long_label() + ": short circuit\n");
      _y0.f1 = OPT::shortckt;
    }
    store_values();
    q_load();
    _m0.c1 = 1./_y0.f1;
    _m0.c0 = _y0.x - _y0.f0 / _y0.f1;
  }else{
    assert(_y0.f0 == LINEAR);
    assert(_y0.f1 == value() || _y0.f1 == OPT::shortckt);
    assert(conchk(_m0.c1, 1./_y0.f1));
    assert(_m0.c0 == 0.);
    assert(_y1 == _y0);
    assert(converged());
  }}
  return converged();
}
/*--------------------------------------------------------------------------*/
void DEV_RESISTANCE::do_ac()
{
  {if (has_ac_eval()){
    ac_eval();
    if (_ev == 0.){
      error(bPICKY, long_label() + ": short circuit\n");
      _ev = OPT::shortckt;
    }
    _acg = 1. / _ev;
  }else{
    assert(_ev == _y0.f1);
    assert(has_tr_eval() || _ev == value());
  }}
  assert(_acg == 1./_ev);
  ac_load();
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
